// File: Multimap.java
// Example of a map with multiple values for each key, and each value is a pair
// Lists all words in any file, and number of occurrences,
//      storing the words by letter

import java.util.*;
import java.io.*;

public class ReadPrintMap {

  // Read the input file name from the user
  public static String GetFileName() throws java.io.IOException
  { DataInputStream FileNameInput=new DataInputStream(System.in);
    System.out.print("File to Open >");
    System.out.flush();
    return(FileNameInput.readLine());
  }

    public static void main(String[] args) throws java.io.IOException
    {   if (args.length!=1) {
          System.err.println("Form: Java ReadPrintMap <Input File Name>");
          System.exit(-1);
        }
        // Associate the disk file with the File object
        System.out.println("Destination file for map:");
        File OutputFile=new File(GetFileName());
        // Now, open a stream from the input file
        PrintWriter DataOutput=
                        new PrintWriter(OutputFile);
        // Read words from file and put into a simulated multimap
        Map<Character, Hashtable<String,Integer>> WordLists
	    = new TreeMap<Character, Hashtable<String,Integer>>();
        try {
	    Scanner s = new Scanner(new File(args[0]));
            while (s.hasNext()) {
		String word = s.next();
		char firstLet=Character.toUpperCase(word.charAt(0));
		if (firstLet>='A' && firstLet <= 'Z') {
                  // get the map for this word, if any
                  Hashtable<String,Integer> Tab=WordLists.get(firstLet);
                  // No entry in WordLists for first letter
                  if (Tab==null) {
                    // Create the table for this letter
                    Tab=new Hashtable<String,Integer>();
                    // Put the word in with a multiplicity of 1
                    Tab.put(word,1);
                    // Put the entry into the map
                    WordLists.put(firstLet,Tab);
                  }
                  else {  // The letter is in the map already
                    // Is the word in the hash table?
                    Integer Count=Tab.get(word);
                    if (Count==null)    // if not, put it in with count of 1
                      Tab.put(word,1);
                    else                // if so, increment its count
                      Tab.put(word,Count+1);
                  }
                }  // if firstLet  
            }
        } catch (IOException e) {
            System.err.println(e);
            System.exit(1);
        }

        // Print all lists
        // First, iterate over all values
        System.out.println("\n**********Lists prefaced by their key**********");
        for (char c : WordLists.keySet())
          System.out.println(c + ": " + WordLists.get(c));

        // Output to file in easily readable fashion. 
        for (char c : WordLists.keySet()) {
          DataOutput.println(c);        // print the letter on a line
          Hashtable<String,Integer> TheTable=WordLists.get(c);        
          DataOutput.println(TheTable.size());
          Iterator it = TheTable.keySet().iterator();
          while (it.hasNext()) {
            String key = (String)it.next();
            Integer value=TheTable.get(key);
            DataOutput.println(key+" "+value);
          }
        }  
        DataOutput.close();
        
        // Now, read it back in from scratch and print it
        // First, we need a Map to read into, and a hash table reference
        Map<Character, Hashtable<String,Integer>> InputLists
	    = new TreeMap<Character, Hashtable<String,Integer>>();
        Hashtable<String,Integer> aTable;        
        // Already have the file object. Now, open a stream from the file
        BufferedReader In=new BufferedReader(new FileReader(OutputFile));
        // Loop through each key, multi-value elemnt in the file        
        String charStr=In.readLine();
        while (charStr!=null)   {       // while not eof
          aTable=new Hashtable<String,Integer>();
          // How many elements in this value? Get it on the next line
          int numKeys=Integer.parseInt(In.readLine());
          for (int count=0;count<numKeys;count++) {
            // Read the words/counter pair
            String line=In.readLine();
            StringTokenizer Tok=new StringTokenizer(line);
            aTable.put(Tok.nextToken(),Integer.valueOf(Tok.nextToken()));
          }
          InputLists.put(charStr.charAt(0),aTable);
          charStr=In.readLine();
        }  
        
        System.out.println(InputLists);  // Just an en masse print
    }
}
